--
-- This is the System ruleset which eventually will provide basic services
-- and access to hardware specific data.
--

module FACTS.Rulesets.System (

	ruleset,
	logEntriesSlot, netTxPowerSlot, netBroadcastSlot

) where

import FACTS.Main

namespace = "system"

ruleset = newRuleset namespace [] [loggingFact, netFact] []

-- Simply append log entries to the "entries" property of this fact.
loggingFact = newFact "logging" [newProperty "entries" (varString "")]

-- This fact has basic network constants: "broadcast" is the broadcast MAC
-- address, "txPower" is to be used for the transmit power parameter of the
-- Send statement. With a setting of 1.5 it makes direct neighbors on a unit
-- grid reachable.
netFact = newFact "net" [newProperty "broadcast" (varInt broadcastMAC), newProperty "txPower" (varFloat 1.5)]

logName = newName namespace "logging"
netName = newName namespace "net"

logEntriesSlot = newSlot logName "entries" []
netBroadcastSlot = newSlot netName "broadcast" []
netTxPowerSlot = newSlot netName "txPower" []
