--
-- This is the Timer ruleset which eventually will provide timing and
-- triggering services.
--

module FACTS.Rulesets.Timer (

	ruleset,
	tickName,
	tickSlot, tackSlot, expiredSlot

) where

import FACTS.Main

namespace = "timer"

ruleset = newRuleset namespace [tickRule, tackRule, expiredRule] [countersFact] []

-- The tick and tack rules react to each other each cycle, at the same time
-- incrementing the tick counter.
tickRule = newRule "tick" maxPriority [condition] [statement1, statement2, statement3]
	where	condition = (Exists tackSlot)
		statement1 = (Retract tackSlot)
		statement2 = (Define tickName [])
		statement3 = (Set countersTicksSlot (BinaryOperation (Constant (varSlot countersTicksSlot)) Add (Constant (varInt 1))))

tackRule = newRule "tack" maxPriority [condition] [statement1, statement2]
	where	condition = (Exists tickSlot)
		statement1 = (Retract tickSlot)
		statement2 = (Define tackName [])

-- As soon as the tick counter reaches a predefined value, this rule creates
-- an expired fact. Based on it other rules may fire.
expiredRule = newRule "expired" maxPriority [condition] [statement1, statement2]
	where	condition = (Eval (Constant (varSlot countersTicksSlot)) Equals (Constant (varInt 10)))
		statement1 = (Set countersTicksSlot (Constant (varInt 0)))
		statement2 = (Define expiredName [])

countersFact = newFact "counters" [newProperty "ticks" (varInt 0)]

countersName = newName namespace "counters"
tickName = newName namespace "tick"
tackName = newName namespace "tack"
expiredName = newName namespace "expired"

countersTicksSlot = newSlot countersName "ticks" []
tickSlot = newSimpleSlot tickName
tackSlot = newSimpleSlot tackName
expiredSlot = newSimpleSlot expiredName