import FACTS.Main
import qualified FACTS.Rulesets.System as System
import qualified FACTS.Rulesets.Timer as Timer

--
-- Problem: Coverage. Shutdown nodes that supply values for regions already covered 
--
-- Create id facts for nodes
--

fact1 = newFact "node" [newProperty "node_id" (varInt 1), newProperty "pos_x" (varFloat 2.0), newProperty "pos_y" (varFloat 2.0)]
fact2 = newFact "node" [newProperty "node_id" (varInt 2), newProperty "pos_x" (varFloat 3.5), newProperty "pos_y" (varFloat 2.0)]
fact3 = newFact "node" [newProperty "node_id" (varInt 3), newProperty "pos_x" (varFloat 3.5), newProperty "pos_y" (varFloat 3.5)]
fact4 = newFact "node" [newProperty "node_id" (varInt 4), newProperty "pos_x" (varFloat 2.0), newProperty "pos_y" (varFloat 3.5)]
fact5 = newFact "node" [newProperty "node_id" (varInt 5), newProperty "pos_x" (varFloat 2.7), newProperty "pos_y" (varFloat 2.7)]

nodeIDSlot = newSlot "node" "node_id" []
nodePosXSlot = newSlot "node" "pos_x" []
nodePosYSlot = newSlot "node" "pos_y" []

rangeFact = newFact "rangeFact" [newProperty "node_id" (varInt 0), newProperty "xmin" (varFloat 0.0), newProperty "xmax" (varFloat 0.0), newProperty "ymin" (varFloat 0.0), newProperty "ymax" (varFloat 0.0)]

rangeNodeIDSlot = newSlot "rangeFact" "node_id" [(Eval (Constant (varSlot (newSlot "rangeFact" "owner" []))) Equals (Constant (varSlot nodeIDSlot)))]
rangeXMinSlot = newSlot "rangeFact" "xmin" [(Eval (Constant (varSlot (newSlot "rangeFact" "owner" []))) Equals (Constant (varSlot nodeIDSlot)))]
rangeXMaxSlot = newSlot "rangeFact" "xmax" [(Eval (Constant (varSlot (newSlot "rangeFact" "owner" []))) Equals (Constant (varSlot nodeIDSlot)))]
rangeYMinSlot = newSlot "rangeFact" "ymin" [(Eval (Constant (varSlot (newSlot "rangeFact" "owner" []))) Equals (Constant (varSlot nodeIDSlot)))]
rangeYMaxSlot = newSlot "rangeFact" "ymax" [(Eval (Constant (varSlot (newSlot "rangeFact" "owner" []))) Equals (Constant (varSlot nodeIDSlot)))]

coveredNodeIDSlot = newSlot "covered" "node_id" []

--
-- First step: Calculate pessimistic range and send it to neighbors
--
rule_range = newRule "range" 50 [condition] [statement1, statement2, statement3, statement4, statement5, statement6, statement7]
	where	condition = (Exists Timer.expiredSlot)
		statement1 = (Retract Timer.expiredSlot)
		statement2 = (Set rangeNodeIDSlot (Constant (varSlot nodeIDSlot)))
		statement3 = (Set rangeXMinSlot (BinaryOperation (Constant (varSlot nodePosXSlot)) Subtract (Constant (varFloat 1.5))))
		statement4 = (Set rangeXMaxSlot (BinaryOperation (Constant (varSlot nodePosXSlot)) Add (Constant (varFloat 1.5))))
		statement5 = (Set rangeYMinSlot (BinaryOperation (Constant (varSlot nodePosYSlot)) Subtract (Constant (varFloat 1.5))))
		statement6 = (Set rangeYMaxSlot (BinaryOperation (Constant (varSlot nodePosYSlot)) Add (Constant (varFloat 1.5))))
		statement7 = (Define "range.computed" [])

--
-- Second step: Send own range to neighbors
--

rule_sendRange = newRule "sendRange" 50 [condition] [statement1, statement2, statement3]
	where	condition = (Exists (newSimpleSlot "range.computed"))
		statement1 = (Retract (newSimpleSlot "range.computed"))
		statement2 = (Send (Constant (varSlot System.netBroadcastSlot)) (Constant (varSlot System.netTxPowerSlot)) (newSlot "rangeFact" "" [(Eval (Constant (varSlot (newSlot "rangeFact" "owner" []))) Equals (Constant (varSlot nodeIDSlot)))]))
		statement3 = (Define "range.send" [])
--
-- for each range received, check if they all cover all of my region
--
-- first: xmin/ymin
--

rule_xymin_covered = newRule "rule_xymin_covered" 30 [condition1, condition2, condition3] [statement1]
	where	condition1 = (Eval (BinaryOperation (Constant (varSlot nodePosXSlot)) Subtract (Constant (varFloat 1.5))) LessThan (Constant (varSlot rangeXMinSlot)))
		condition2 = (Eval (BinaryOperation (Constant (varSlot nodePosYSlot)) Subtract (Constant (varFloat 1.5))) LessThan (Constant (varSlot rangeYMinSlot)))
		condition3 = (Exists (newSimpleSlot "range.send"))
		statement1 = (Define "xmin_ymin_covered" [("node_id", Constant (varSlot nodeIDSlot))])


--
-- second: xmax/ymin

rule_xmaxymin_covered = newRule "rule_xmaxymin_covered" 30 [condition1, condition2, condition3] [statement1]
	where	condition1 = (Eval (BinaryOperation (Constant (varSlot nodePosXSlot)) Add (Constant (varFloat 1.5))) GreaterThan (Constant (varSlot rangeXMaxSlot)))
		condition2 = (Eval (BinaryOperation (Constant (varSlot nodePosYSlot)) Subtract (Constant (varFloat 1.5))) LessThan (Constant (varSlot rangeYMinSlot)))
		condition3 = (Exists (newSimpleSlot "range.send"))
		statement1 = (Define "xmax_ymin_covered" [("node_id", Constant (varSlot nodeIDSlot))])

--
--third: xmax/ymax
--

rule_xymax_covered = newRule "rule_xymax_covered" 30 [condition1, condition2, condition3] [statement1]
	where	condition1 = (Eval (BinaryOperation (Constant (varSlot nodePosXSlot)) Add (Constant (varFloat 1.5))) GreaterThan (Constant (varSlot rangeXMaxSlot)))
		condition2 = (Eval (BinaryOperation (Constant (varSlot nodePosYSlot)) Add (Constant (varFloat 1.5))) GreaterThan (Constant (varSlot rangeYMaxSlot)))
		condition3 = (Exists (newSimpleSlot "range.send"))
		statement1 = (Define "xmax_ymax_covered" [("node_id", Constant (varSlot nodeIDSlot))])

--fourth: xmin/ymax
--

rule_xminymax_covered = newRule "rule_xminymax_covered" 30 [condition1, condition2, condition3] [statement1]
	where	condition1 = (Eval (BinaryOperation (Constant (varSlot nodePosXSlot)) Subtract (Constant (varFloat 1.5))) LessThan (Constant (varSlot rangeXMinSlot)))
		condition2 = (Eval (BinaryOperation (Constant (varSlot nodePosYSlot)) Add (Constant (varFloat 1.5))) GreaterThan (Constant (varSlot rangeYMaxSlot)))
		condition3 = (Exists (newSimpleSlot "range.send"))
		statement1 = (Define "xmin_ymax_covered" [("node_id", Constant (varSlot nodeIDSlot))])


-- 
-- if all parts are covered, than the node can potentially be switched off
--

rule_determine_coverage = newRule "rule_determine_coverage" 30 [condition1, condition2, condition3, condition4][statement1]
	where	condition1 = (Exists (newSimpleSlot "xmin_ymin_covered"))
		condition2 = (Exists (newSimpleSlot "xmax_ymin_covered"))
		condition3 = (Exists (newSimpleSlot "xmax_ymax_covered"))
		condition4 = (Exists (newSimpleSlot "xmin_ymax_covered"))
		statement1 = (Define "covered" [("node_id", Constant (varSlot nodeIDSlot))])


--
-- if node realizes it is covered (covered.id == self), it sends a message to its neighbors that it is going to switch itself off
-- so they retract the range fact of this node
--


rule_off = newRule "rule_off" 50 [condition1, condition2] [statement1, statement2, statement3]
	where	condition1 = (Exists (newSimpleSlot "covered"))
		condition2 = (Eval (BinaryOperation (Constant (varSlot coveredNodeIDSlot)) Subtract (Constant (varSlot nodeIDSlot))) Equals (Constant (varInt 0)))
		statement1 = (Send (Constant (varSlot System.netBroadcastSlot)) (Constant (varSlot System.netTxPowerSlot)) (newSimpleSlot "covered"))
		statement2 = (Retract (newSimpleSlot "covered"))
		statement3 = (Define "OFF" [])
		
--
-- if node gets a covered fact from some other node, it has to check its bounds, therefore produce for every bound a fact to communicate this
--		
		
rule_check = newRule "rule_check" 70 [condition1, condition2] [statement1, statement2, statement3, statement4, statement5]
	where	condition1 = (Exists (newSimpleSlot "covered"))
		condition2 = (Eval (BinaryOperation (Constant (varSlot coveredNodeIDSlot)) Subtract (Constant (varSlot nodeIDSlot))) NotEquals (Constant (varInt 0)))
		statement1 = (Define "check_minmin" [("node_id", Constant (varSlot coveredNodeIDSlot))])
		statement2 = (Define "check_minmax" [("node_id", Constant (varSlot coveredNodeIDSlot))])
		statement3 = (Define "check_maxmin" [("node_id", Constant (varSlot coveredNodeIDSlot))])
		statement4 = (Define "check_maxmax" [("node_id", Constant (varSlot coveredNodeIDSlot))])
		statement5 = (Retract (newSimpleSlot "covered"))
		
--
-- check every bound whether there is an overlap
--		

rule_recvCov1a = newRule "rule_recvCov1a" 60 [condition1, condition2] [statement1, statement2]
	where	condition1 = (Exists (newSimpleSlot "checkminmin"))
		condition2 = (Eval (BinaryOperation (Constant (varSlot (newSlot "check_minmin" "node_id" []))) Subtract (Constant (varSlot (newSlot "xmin_ymin_covered" "node_id" [])))) Equals (Constant (varInt 0)))
		statement1 = (Retract (newSimpleSlot "checkminmin"))
		statement2 = (Retract (newSimpleSlot "xmin_ymin_covered"))
		
rule_recvCov1b = newRule "rule_recvCov1b" 50 [condition1, condition2] [statement1]
	where	condition1 = (Exists (newSimpleSlot "checkminmin"))
		condition2 = (Eval (BinaryOperation (Constant (varSlot (newSlot "check_minmin" "node_id" []))) Subtract (Constant (varSlot (newSlot "xmin_ymin_covered" "node_id" [])))) NotEquals (Constant (varInt 0)))
		statement1 = (Retract (newSimpleSlot "checkminmin"))
		
		
rule_recvCov2a = newRule "rule_recvCov2a" 60 [condition1, condition2] [statement1, statement2]
	where	condition1 = (Exists (newSimpleSlot "checkminmax"))
		condition2 = (Eval (BinaryOperation (Constant (varSlot (newSlot "check_minmax" "node_id" []))) Subtract (Constant (varSlot (newSlot "xmin_ymax_covered" "node_id" [])))) Equals (Constant (varInt 0)))
		statement1 = (Retract (newSimpleSlot "checkminmax"))
		statement2 = (Retract (newSimpleSlot "xmin_ymax_covered"))
		
rule_recvCov2b = newRule "rule_recvCov2b" 50 [condition1, condition2] [statement1]
	where	condition1 = (Exists (newSimpleSlot "checkminmax"))
		condition2 = (Eval (BinaryOperation (Constant (varSlot (newSlot "check_minmax" "node_id" []))) Subtract (Constant (varSlot (newSlot "xmin_ymax_covered" "node_id" [])))) NotEquals (Constant (varInt 0)))
		statement1 = (Retract (newSimpleSlot "checkminmax"))
		
		
rule_recvCov3a = newRule "rule_recvCov3a" 60 [condition1, condition2] [statement1, statement2]
	where	condition1 = (Exists (newSimpleSlot "checkmaxmin"))
		condition2 = (Eval (BinaryOperation (Constant (varSlot (newSlot "check_maxmin" "node_id" []))) Subtract (Constant (varSlot (newSlot "xmax_ymin_covered" "node_id" [])))) Equals (Constant (varInt 0)))
		statement1 = (Retract (newSimpleSlot "checkmaxmin"))
		statement2 = (Retract (newSimpleSlot "xmax_ymin_covered"))
		
rule_recvCov3b = newRule "rule_recvCov3b" 50 [condition1, condition2] [statement1]
	where	condition1 = (Exists (newSimpleSlot "checkmaxmin"))
		condition2 = (Eval (BinaryOperation (Constant (varSlot (newSlot "check_maxmin" "node_id" []))) Subtract (Constant (varSlot (newSlot "xmax_ymin_covered" "node_id" [])))) NotEquals (Constant (varInt 0)))
		statement1 = (Retract (newSimpleSlot "checkmaxmin"))
		
		
rule_recvCov4a = newRule "rule_recvCov4a" 60 [condition1, condition2] [statement1, statement2]
	where	condition1 = (Exists (newSimpleSlot "checkmaxmax"))
		condition2 = (Eval (BinaryOperation (Constant (varSlot (newSlot "check_maxmax" "node_id" []))) Subtract (Constant (varSlot (newSlot "xmax_ymax_covered" "node_id" [])))) Equals (Constant (varInt 0)))
		statement1 = (Retract (newSimpleSlot "checkmaxmax"))
		statement2 = (Retract (newSimpleSlot "xmax_ymax_covered"))
		
rule_recvCov4b = newRule "rule_recvCov4b" 50 [condition1, condition2] [statement1]
	where	condition1 = (Exists (newSimpleSlot "checkmaxmax"))
		condition2 = (Eval (BinaryOperation (Constant (varSlot (newSlot "check_maxmax" "node_id" []))) Subtract (Constant (varSlot (newSlot "xmax_ymax_covered" "node_id" [])))) NotEquals (Constant (varInt 0)))
		statement1 = (Retract (newSimpleSlot "checkmaxmax"))




node1 = addRulesets (newNode 1 (2.0,2.0)) [System.ruleset, Timer.ruleset]
node1a = addFacts node1 [fact1, rangeFact]
node1b = addRules node1a [rule_range, rule_sendRange, rule_xymin_covered, rule_xmaxymin_covered, rule_xymax_covered, rule_xminymax_covered, rule_determine_coverage]

node2 = addRulesets (newNode 2 (3.5,2.0)) [System.ruleset, Timer.ruleset]
node2a = addFacts node2 [fact2, rangeFact]
node2b = addRules node2a [rule_range, rule_sendRange, rule_xymin_covered, rule_xmaxymin_covered, rule_xymax_covered, rule_xminymax_covered, rule_determine_coverage, rule_off, rule_check, rule_recvCov1a, rule_recvCov1b, rule_recvCov2a, rule_recvCov2b, rule_recvCov3a, rule_recvCov3b, rule_recvCov4a, rule_recvCov4b]

node3 = addRulesets (newNode 3 (3.5,3.5)) [System.ruleset, Timer.ruleset]
node3a = addFacts node3 [fact3, rangeFact]
node3b = addRules node3a [rule_range, rule_sendRange, rule_xymin_covered, rule_xmaxymin_covered, rule_xymax_covered, rule_xminymax_covered, rule_determine_coverage, rule_off, rule_check, rule_recvCov1a, rule_recvCov1b, rule_recvCov2a, rule_recvCov2b, rule_recvCov3a, rule_recvCov3b, rule_recvCov4a, rule_recvCov4b]

node4 = addRulesets (newNode 4 (2.0,3.5)) [System.ruleset, Timer.ruleset]
node4a = addFacts node4 [fact4, rangeFact]
node4b = addRules node4a [rule_range, rule_sendRange, rule_xymin_covered, rule_xmaxymin_covered, rule_xymax_covered, rule_xminymax_covered, rule_determine_coverage, rule_off, rule_check, rule_recvCov1a, rule_recvCov1b, rule_recvCov2a, rule_recvCov2b, rule_recvCov3a, rule_recvCov3b, rule_recvCov4a, rule_recvCov4b]

node5 = addRulesets (newNode 5 (2.7,2.7)) [System.ruleset, Timer.ruleset]
node5a = addFacts node5 [fact5, rangeFact]
node5b = addRules node5a [rule_range, rule_sendRange, rule_xymin_covered, rule_xmaxymin_covered, rule_xymax_covered, rule_xminymax_covered, rule_determine_coverage, rule_off, rule_check, rule_recvCov1a, rule_recvCov1b, rule_recvCov2a, rule_recvCov2b, rule_recvCov3a, rule_recvCov3b, rule_recvCov4a, rule_recvCov4b]

network = addNodes newNetwork [node1b, node2b, node3b, node4b, node5b]

startTimerEvent = newEvent 0 0 [(Define Timer.tickName [])]
events = [startTimerEvent]

sim :: Int -> IO()
sim state = putStr(show ((runSimulation (newSimulation network events))!!state))
