--
-- This file is for debugging and testing purposes only.
--

-- Import FACTS and some rulesets.
import FACTS.Main
import qualified FACTS.Rulesets.System as System
import qualified FACTS.Rulesets.Timer as Timer

-- Transmit the "Timer.tick" to the second node once the timer expires.
-- The second node will start counting until its timer eventually expires, too.
transmitTickRule = newRule "transmitTick" minPriority [condition] [statement1, statement2, statement3]
	where	condition = (Exists Timer.expiredSlot)
		statement1 = (Define Timer.tickName [])
		statement2 = (Send (Constant (varSlot System.netBroadcastSlot)) (Constant (varSlot System.netTxPowerSlot)) Timer.tickSlot)
		statement3 = (Retract Timer.tickSlot)

-- Test aggregation statements "Sum" and "Count".
aggregateRule = newRule "aggregateRule" minPriority [condition] [statement1, statement2]
	where	condition = (Exists (newSimpleSlot "aggregate"))
		statement1 = (Set (newSlot "aggregate" "value" []) (UnaryOperation Sum (Constant (varSlot dataValueSlot))))
		statement2 = (Set (newSlot "aggregate" "count" []) (UnaryOperation Count (Constant (varSlot dataValueSlot))))
		dataValueSlot = newSlot "data" "value" []

-- Data points to be aggregated.
dataFact1 = newFact "data" [newProperty "value" (varInt 1)]
dataFact2 = newFact "data" [newProperty "value" (varInt 2)]
dataFact3 = newFact "data" [newProperty "value" (varInt 3)]

-- Create two nodes, one timer-only, one with aggregration.
timerNode1 = addRulesets (newNode 1 (0,0)) [System.ruleset, Timer.ruleset]
timerNode1r = addRules timerNode1 [transmitTickRule]
timerNode1rf = addFacts timerNode1r []

timerNode2 = addRulesets (newNode 2 (0,1)) [System.ruleset, Timer.ruleset]
timerNode2r = addRules timerNode2 [transmitTickRule, aggregateRule]
timerNode2rf = addFacts timerNode2r [dataFact1, dataFact2, dataFact3]

-- Create network.
network = addNodes newNetwork [timerNode1rf, timerNode2rf]

-- Add timer and aggregation events to the simulation.
startTimerEvent = newEvent 6 1 [(Define Timer.tickName [])]
aggregateEvent = newEvent 3 2 [(Define "aggregate" [("value", (Constant (varInt 0)))])]
events = [startTimerEvent, aggregateEvent]

-- Initialize simulation and display results a particular state.
sim :: Int -> IO()
sim state = putStr(show ((runSimulation (newSimulation network events))!!state))
