import FACTS.Main
import qualified FACTS.Rulesets.System as System
import qualified FACTS.Rulesets.Timer as Timer

--
-- Problem: Directed Diffusion, Routing Algorithm for WSN
--
-- Create nodes
--

fact1 = newFact "node" [newProperty "pos_x" (varFloat 2.0), newProperty "pos_y" (varFloat 2.0)]
fact2 = newFact "node" [newProperty "pos_x" (varFloat 3.5), newProperty "pos_y" (varFloat 2.0)]
fact3 = newFact "node" [newProperty "pos_x" (varFloat 3.5), newProperty "pos_y" (varFloat 3.5)]
fact4 = newFact "node" [newProperty "pos_x" (varFloat 2.0), newProperty "pos_y" (varFloat 3.5)]
fact5 = newFact "node" [newProperty "pos_x" (varFloat 2.7), newProperty "pos_y" (varFloat 2.7)]
fact6 = newFact "node" [newProperty "pos_x" (varFloat 5.0), newProperty "pos_y" (varFloat 1.5)]
fact7 = newFact "node" [newProperty "pos_x" (varFloat 4.0), newProperty "pos_y" (varFloat 1.0)]
fact8 = newFact "node" [newProperty "pos_x" (varFloat 5.0), newProperty "pos_y" (varFloat 3.0)]
fact9 = newFact "node" [newProperty "pos_x" (varFloat 4.5), newProperty "pos_y" (varFloat 3.5)]
fact10 = newFact "node" [newProperty "pos_x" (varFloat 6.0), newProperty "pos_y" (varFloat 3.0)]

sinkFact = newFact "sink" []
--
-- First step: Start dissemination protocol from sink, sink == node 1
--
sink_assign = newRule "sink_assign" 50 [condition1, condition2] [statement1, statement2, statement3, statement4, statement5]
	where	condition1 = Exists Timer.expiredSlot
		condition2 = Exists (newSimpleSlot "sink")
		statement1 = Retract Timer.expiredSlot
		statement2 = Define "interest" []
		statement3 = Set (newSlot "interest" "sink" []) (Constant (varSlot (newSlot "node" "owner" [])))
		statement4 = Set (newSlot "interest" "sender" []) (Constant (varSlot (newSlot "node" "owner" [])))
		statement5 = Set (newSlot "interest" "hops" []) (Constant (varInt 0))



node1 = addRulesets (newNode 1 (2.0,2.0)) [System.ruleset, Timer.ruleset]
node1a = addFacts node1 [fact1, sinkFact]
node1b = addRules node1a [sink_assign]

node2 = addRulesets (newNode 2 (3.5,2.0)) [System.ruleset, Timer.ruleset]
node2a = addFacts node2 [fact2]
node2b = addRules node2a []

node3 = addRulesets (newNode 3 (3.5,3.5)) [System.ruleset, Timer.ruleset]
node3a = addFacts node3 [fact3]
node3b = addRules node3a []

node4 = addRulesets (newNode 4 (2.0,3.5)) [System.ruleset, Timer.ruleset]
node4a = addFacts node4 [fact4]
node4b = addRules node4a []

node5 = addRulesets (newNode 5 (2.7,2.7)) [System.ruleset, Timer.ruleset]
node5a = addFacts node5 [fact5]
node5b = addRules node5a []

node6 = addRulesets (newNode 6 (5.0,1.5)) [System.ruleset, Timer.ruleset]
node6a = addFacts node6 [fact6]
node6b = addRules node6a []

node7 = addRulesets (newNode 7 (4.0,1.0)) [System.ruleset, Timer.ruleset]
node7a = addFacts node7 [fact7]
node7b = addRules node7a []

node8 = addRulesets (newNode 8 (5.0,3.0)) [System.ruleset, Timer.ruleset]
node8a = addFacts node8 [fact8]
node8b = addRules node8a []

node9 = addRulesets (newNode 9 (4.5,3.5)) [System.ruleset, Timer.ruleset]
node9a = addFacts node9 [fact9]
node9b = addRules node9a []

node10 = addRulesets (newNode 10 (6.0,3.0)) [System.ruleset, Timer.ruleset]
node10a = addFacts node10 [fact10]
node10b = addRules node10a []

network = addNodes newNetwork [node1b, node2b, node3b, node4b, node5b, node6b, node7b, node8b, node9b, node10b]

startTimerEvent = newEvent 0 0 [(Define Timer.tickName [])]
events = [startTimerEvent]

sim :: Int -> IO()
sim state = putStr(show ((runSimulation (newSimulation network events))!!state))
