import FACTS.Main
import qualified FACTS.Rulesets.System as System
import qualified FACTS.Rulesets.Timer as Timer

--
-- Create id facts for nodes
--

node1Fact = newFact "node" [newProperty "node_id" (varInt 1)]
node2Fact = newFact "node" [newProperty "node_id" (varInt 2)]
node3Fact = newFact "node" [newProperty "node_id" (varInt 3)]

nodeIDSlot = newSlot "node" "node_id" []

aggregationFact = newFact "netAgg" [newProperty "value" (varFloat 0.0)]

--
-- Create rule to read temperature sensor on expired event
--

rule_temp_read1 = newRule "TempRead1" 20 [condition1, condition2] [statement1, statement2, statement3, statement4]
	where	condition1 = (Exists Timer.expiredSlot)
		condition2 = (Eval (Constant (varSlot nodeIDSlot)) Equals (Constant (varInt 1)))
		statement1 = (Define "temp_read" [("tempVal", Constant (varFloat 25.0))])
		statement2 = (Retract Timer.expiredSlot)
		statement3 = (Define "local_read" [])
		statement4 = (Set System.logEntriesSlot (BinaryOperation (Constant (varSlot System.logEntriesSlot)) Add (Constant (varString "setting temp = 25, "))))

rule_temp_read2 = newRule "TempRead2" 20 [condition1, condition2] [statement1, statement2, statement3, statement4]
	where	condition1 = (Exists Timer.expiredSlot)
		condition2 = (Eval (Constant (varSlot nodeIDSlot)) Equals (Constant (varInt 2)))
		statement1 = (Define "temp_read" [("tempVal", Constant (varFloat 28.0))])
		statement2 = (Retract Timer.expiredSlot)
		statement3 = (Define "local_read" [])
		statement4 = (Set System.logEntriesSlot (BinaryOperation (Constant (varSlot System.logEntriesSlot)) Add (Constant (varString "setting temp = 28, "))))

rule_temp_read3 = newRule "TempRead3" 20 [condition1, condition2] [statement1, statement2, statement3, statement4]
	where	condition1 = (Exists Timer.expiredSlot)
		condition2 = (Eval (Constant (varSlot nodeIDSlot)) Equals (Constant (varInt 3)))
		statement1 = (Define "temp_read" [("tempVal", Constant (varFloat 31.0))])
		statement2 = (Retract Timer.expiredSlot)
		statement3 = (Define "local_read" [])
		statement4 = (Set System.logEntriesSlot (BinaryOperation (Constant (varSlot System.logEntriesSlot)) Add (Constant (varString "setting temp = 31, "))))

--
-- As soon as locally sampled TempVal is fact in rule engine, send it to neighbors
--

rule_aggValues = newRule "aggValues" 20 [condition1, condition2] [statement1, statement2, statement3]
	where	condition1 = (Exists (newSimpleSlot "temp_read"))
		condition2 = (Exists (newSimpleSlot "local_read"))
		statement1 = (Send (Constant (varSlot System.netBroadcastSlot)) (Constant (varSlot System.netTxPowerSlot)) (newSimpleSlot "temp_read"))
		statement2 = (Retract (newSimpleSlot "local_read"))
		statement3 = (Set System.logEntriesSlot (BinaryOperation (Constant (varSlot System.logEntriesSlot)) Add (Constant (varString "sending temp, "))))


node1 = addRulesets (newNode 1 (0,0)) [System.ruleset, Timer.ruleset]
node1r = addRules node1 [rule_temp_read1, rule_temp_read2, rule_temp_read3]
node1rf = addFacts node1r [node1Fact]

node2 = addRulesets (newNode 2 (1.5,0)) [System.ruleset, Timer.ruleset]
node2r = addRules node2 [rule_temp_read1, rule_temp_read2, rule_temp_read3]
node2rf = addFacts node2r [node2Fact]

node3 = addRulesets (newNode 3 (0,1.5)) [System.ruleset, Timer.ruleset]
node3r = addRules node3 [rule_temp_read1, rule_temp_read2, rule_temp_read3]
node3rf = addFacts node3r [node3Fact]

network = addNodes newNetwork [node1rf, node2rf, node3rf]

startTimerEvent = newEvent 0 0 [(Define Timer.tickName [])]
events = [startTimerEvent]

sim :: Int -> IO()
sim state = putStr(show ((runSimulation (newSimulation network events))!!state))