--
-- This script implements a Turing Machine that increments a binary-coded
-- number on the tape and then stops.
--
-- See Uwe Schning "Theoretische Informatik -- kurzgefasst" p. 83
--

import FACTS.Main
import qualified FACTS.Rulesets.TuringMachine as TM

-- Define constants just as in the text book.
z0 = varInt 0
z1 = varInt 1
z2 = varInt 2
zE = (varSlot TM.constErrorStateSlot)

s0 = varString "0"
s1 = varString "1"
sB = (varSlot TM.constBlankSymbolSlot)

-- These are just short-hand notation to make the function table more readable.
r = (varSlot TM.constRightSlot)
l = (varSlot TM.constLeftSlot)
n = (varSlot TM.constNeutralSlot)

-- These facts define the function of the Turing Machine, matching current
-- state and symbol to the next symbol, state, and tape movement.
functionFact1 = newFact TM.functionName [newProperty "state" z0, newProperty "symbol" s0, newProperty "nextState" z0, newProperty "nextSymbol" s0, newProperty "nextMovement" r]
functionFact2 = newFact TM.functionName [newProperty "state" z0, newProperty "symbol" s1, newProperty "nextState" z0, newProperty "nextSymbol" s1, newProperty "nextMovement" r]
functionFact3 = newFact TM.functionName [newProperty "state" z0, newProperty "symbol" sB, newProperty "nextState" z1, newProperty "nextSymbol" sB, newProperty "nextMovement" l]
functionFact4 = newFact TM.functionName [newProperty "state" z1, newProperty "symbol" s0, newProperty "nextState" z2, newProperty "nextSymbol" s1, newProperty "nextMovement" l]
functionFact5 = newFact TM.functionName [newProperty "state" z1, newProperty "symbol" s1, newProperty "nextState" z1, newProperty "nextSymbol" s0, newProperty "nextMovement" l]
functionFact6 = newFact TM.functionName [newProperty "state" z1, newProperty "symbol" sB, newProperty "nextState" zE, newProperty "nextSymbol" s1, newProperty "nextMovement" n]
functionFact7 = newFact TM.functionName [newProperty "state" z2, newProperty "symbol" s0, newProperty "nextState" z2, newProperty "nextSymbol" s0, newProperty "nextMovement" l]
functionFact8 = newFact TM.functionName [newProperty "state" z2, newProperty "symbol" s1, newProperty "nextState" z2, newProperty "nextSymbol" s1, newProperty "nextMovement" l]
functionFact9 = newFact TM.functionName [newProperty "state" z2, newProperty "symbol" sB, newProperty "nextState" zE, newProperty "nextSymbol" sB, newProperty "nextMovement" r]

-- These facts are the content of the tape.
tapeFact1 = newFact TM.tapeName [newProperty "position" (varInt 1), newProperty "symbol" s1]
tapeFact2 = newFact TM.tapeName [newProperty "position" (varInt 2), newProperty "symbol" s0]
tapeFact3 = newFact TM.tapeName [newProperty "position" (varInt 3), newProperty "symbol" s1]

-- Initialize one node with the Turing Machine ruleset, the function facts, and
-- the tape facts.
node1 = addRulesets (newNode 1 (0,0)) [TM.ruleset]
node1f = addFacts node1 [functionFact1, functionFact2, functionFact3, functionFact4, functionFact5, functionFact6, functionFact7, functionFact8, functionFact9]
node1fb = addFacts node1f [tapeFact1, tapeFact2, tapeFact3]

-- Add node to the network.
network = addNodes newNetwork [node1fb]

-- Create an event that defines the initial state of the simulation and hence
-- of the Turing Mashine.
startEvent = newEvent 0 0 [
	(Define TM.runName [
		("state", Constant z0),
		("position", Constant (varInt 1)),
		("leftBorder", Constant (varInt 1)),
		("rightBorder", Constant (varInt 3))])]
events = [startEvent]

-- Initialize simulation and display results.
sim :: Int -> IO()
sim state = putStr(show ((runSimulation (newSimulation network events))!!state))
